@@ -142,5 +142,17 @@ class HyperVClusterException(HyperVException):
     pass
 
 
+class DNSException(OSWinException):
+    pass
+
+
+class DNSZoneNotFound(NotFound, DNSException):
+    msg_fmt = _("DNS Zone not found: %(zone_name)s")
+
+
+class DNSZoneAlreadyExists(DNSException):
+    msg_fmt = _("DNS Zone already exists: %(zone_name)s")
+
+
 class JobTerminateFailed(HyperVException):
     msg_fmt = _("Could not terminate the requested job(s).")
