@@ -52,7 +52,9 @@ class NamedPipeTestCase(base.BaseTestCase):
     def _mock_setup_pipe_handler(self):
         self._handler._log_file_handle = mock.Mock()
         self._handler._pipe_handle = mock.sentinel.pipe_handle
-        self._handler._workers = [mock.Mock(), mock.Mock()]
+        self._r_worker = mock.Mock()
+        self._w_worker = mock.Mock()
+        self._handler._workers = [self._r_worker, self._w_worker]
         self._handler._r_buffer = mock.Mock()
         self._handler._w_buffer = mock.Mock()
         self._handler._r_overlapped = mock.Mock()
@@ -90,24 +92,58 @@ class NamedPipeTestCase(base.BaseTestCase):
 
         mock_stop_handler.assert_called_once_with()
 
-    @mock.patch.object(namedpipe.NamedPipeHandler, '_close_pipe')
-    def test_stop_pipe_handler(self, mock_close_pipe):
+    @mock.patch.object(namedpipe.NamedPipeHandler, '_cleanup_handles')
+    @mock.patch.object(namedpipe.NamedPipeHandler, '_cancel_io')
+    def _test_stop_pipe_handler(self, mock_cancel_io,
+                                mock_cleanup_handles,
+                                workers_started=True):
         self._mock_setup_pipe_handler()
+        if not workers_started:
+            handler_workers = []
+            self._handler._workers = handler_workers
+        else:
+            handler_workers = self._handler._workers
+            self._r_worker.is_alive.side_effect = (True, False)
+            self._w_worker.is_alive.return_value = False
 
         self._handler.stop()
 
         self._handler._stopped.set.assert_called_once_with()
+        if not workers_started:
+            mock_cleanup_handles.assert_called_once_with()
+        else:
+            self.assertFalse(mock_cleanup_handles.called)
+
+        if workers_started:
+            mock_cancel_io.assert_called_once_with()
+            self._r_worker.join.assert_called_once_with(0.5)
+            self.assertFalse(self._w_worker.join.called)
+
+        self.assertEqual([], self._handler._workers)
+
+    def test_stop_pipe_handler_workers_started(self):
+        self._test_stop_pipe_handler()
+
+    def test_stop_pipe_handler_workers_not_started(self):
+        self._test_stop_pipe_handler(workers_started=False)
+
+    @mock.patch.object(namedpipe.NamedPipeHandler, '_close_pipe')
+    def test_cleanup_handles(self, mock_close_pipe):
+        self._mock_setup_pipe_handler()
+        log_handle = self._handler._log_file_handle
+        r_event = self._handler._r_overlapped.hEvent
+        w_event = self._handler._w_overlapped.hEvent
+
+        self._handler._cleanup_handles()
+
         mock_close_pipe.assert_called_once_with()
-        self._handler._log_file_handle.close.assert_called_once_with()
+        log_handle.close.assert_called_once_with()
+        self._ioutils.close_handle.assert_has_calls(
+            [mock.call(r_event), mock.call(w_event)])
 
-        self._ioutils.set_event.assert_has_calls(
-            [mock.call(self._handler._r_overlapped.hEvent),
-             mock.call(self._handler._w_overlapped.hEvent)])
-        self._ioutils.cancel_io.assert_called_once_with(
-            mock.sentinel.pipe_handle)
-
-        for worker in self._handler._workers:
-            worker.join.assert_called_once_with()
+        self.assertIsNone(self._handler._log_file_handle)
+        self.assertIsNone(self._handler._r_overlapped.hEvent)
+        self.assertIsNone(self._handler._w_overlapped.hEvent)
 
     def test_setup_io_structures(self):
         self._handler._setup_io_structures()
@@ -165,11 +201,14 @@ class NamedPipeTestCase(base.BaseTestCase):
 
         self._handler._cancel_io()
 
-        self._ioutils.set_event.assert_has_calls(
-            [mock.call(self._handler._r_overlapped.hEvent),
-             mock.call(self._handler._w_overlapped.hEvent)])
-        self._ioutils.cancel_io.assert_called_once_with(
-            mock.sentinel.pipe_handle)
+        overlapped_structures = [self._handler._r_overlapped,
+                                 self._handler._w_overlapped]
+
+        self._ioutils.cancel_io.assert_has_calls(
+            [mock.call(self._handler._pipe_handle,
+                       overlapped_structure,
+                       ignore_invalid_handle=True)
+             for overlapped_structure in overlapped_structures])
 
     @mock.patch.object(namedpipe.NamedPipeHandler, '_start_io_worker')
     def test_read_from_pipe(self, mock_start_worker):
@@ -196,7 +235,9 @@ class NamedPipeTestCase(base.BaseTestCase):
             self._handler._w_completion_routine,
             self._handler._get_data_to_write)
 
-    def _test_start_io_worker(self, buff_update_func=None, exception=None):
+    @mock.patch.object(namedpipe.NamedPipeHandler, '_cleanup_handles')
+    def _test_start_io_worker(self, mock_cleanup_handles,
+                              buff_update_func=None, exception=None):
         self._handler._stopped.isSet.side_effect = [False, True]
         self._handler._pipe_handle = mock.sentinel.pipe_handle
         self._handler.stop = mock.Mock()
@@ -218,8 +259,10 @@ class NamedPipeTestCase(base.BaseTestCase):
                                         fake_buffer, num_bytes,
                                         mock.sentinel.overlapped_structure,
                                         mock.sentinel.completion_routine)
+
         if exception:
             self._handler._stopped.set.assert_called_once_with()
+        mock_cleanup_handles.assert_called_once_with()
 
     def test_start_io_worker(self):
         self._test_start_io_worker()
